#!/usr/bin/env node

/*
Copyright (c) 2018-2019 Uber Technologies, Inc.

This source code is licensed under the MIT license found in the
LICENSE file in the root directory of this source tree.
*/
"use strict";

var _process = _interopRequireDefault(require("process"));

var _yargs = _interopRequireDefault(require("yargs"));

var _chalk = _interopRequireDefault(require("chalk"));

var _core = require("@dubstep/core");

var _styledV7TypeArguments = _interopRequireDefault(require("./styled-v7-type-arguments.js"));

var _styledV8ThemedStyled = _interopRequireDefault(require("./styled-v8-themedStyled.js"));

var _v9ThemeScale = _interopRequireDefault(require("./v9-theme-scale.js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Each mod or script is an array of dubstep steps
// Users can select one of these to run from the CLI via `--mod`
const MODS = {
  v8Types: [_step('Apply type arguments to baseui "styled" calls.', _styledV7TypeArguments.default), _step('Migrate styled theme generic to "createThemedStyled".', _styledV8ThemedStyled.default)],
  v9Styles: [_step('Map colors and typography to v9. These changes are not idempotent!', _v9ThemeScale.default)]
}; // Wrapper for step
// Mainly so that we can log each step, which dubstep doesn't do

function _step(msg, fn) {
  return (0, _core.step)(msg, async () => {
    console.log(_chalk.default.dim(`⮑  ${msg}`));
    await fn({
      dir
    });
  });
} // Set up yargs
// Includes usage on -h or --help


const {
  argv
} = _yargs.default.usage(`${_chalk.default.white.bold('baseui-codemods')}

You can use this CLI tool to run various scripts (codemods) that will automate laborious migrations between major versions of the "baseui" library.

Usage:
  $ baseui-codemods --dir=<PATH_TO_CODE> --mod=<CODEMOD_NAME>

Example:
  $ baseui-codemods --dir=src --mod=v8Types

Codemods:
${Object.keys(MODS).map(m => `  ${m}`).join(`\n`)}`).epilogue(`Visit ${_chalk.default.bold('https://baseweb.design/blog')} for more information on "baseui" and details on major version migrations.`).option('dir', {
  description: 'The directory to run the codemod on'
}).option('mod', {
  description: 'The name of the codemod to run'
}).help().alias('help', 'h'); // Get `dir` parameter


let dir = argv.dir;

if (!dir) {
  console.log(_chalk.default.white.bold('baseui-codemods'));
  console.error(`${_chalk.default.red('error')} Must specify a directory with the "--dir" argument. Ex: "--dir=src".`);

  _process.default.exit(1);
} // terminal auto-completes directories with a '/' suffix


if (dir[dir.length - 1] === '/') {
  dir = dir.substring(0, dir.length - 1);
} // Get `mod` parameter


const mod = argv.mod;

if (!mod || !Object.keys(MODS).includes(mod)) {
  console.error(`${_chalk.default.red('error')} Must specify a valid codemod to run with the "--mod" argument. Here are the currently available mods:${Object.keys(MODS).map(m => `\n  --mod=${m}`).join('')}`);

  _process.default.exit(1);
} // Retrieve steps for codemod


const steps = MODS[mod]; // Add a finish step to script for logging completion

const start = _process.default.hrtime();

steps.push((0, _core.step)('finish', async () => {
  const end = _process.default.hrtime(start);

  console.log(`Successfully completed running ${_chalk.default.cyan(mod)} codemod.`);
  console.log(`✨  Done in ${(end[0] + end[1] / 1e9).toFixed(2)}s.`);
})); // We are ready to codemod!

console.log(`${_chalk.default.white.bold('baseui-codemods')} ${_chalk.default.cyan(mod)}`); // Run selected codemod

const stepper = new _core.Stepper(steps);
stepper.run();